/*++

Copyright (c) 2003 Microsoft Corporation All Rights Reserved

Module Name:

    mxe_drv.h

Abstract:

    This module contains the common private declarations
    for the Mxe Bus enumerator.

Environment:

    kernel mode only

--*/

#pragma once

#define BUSENUM_POOL_TAG (ULONG) 'suBT'
#define N_BARS		3

#include "net\mlx4.h"
#include <mlx4_debug.h>
#include "bus_intf.h"
#include "fc_bus_if.h"

#if DBG
#define BUS_DEFAULT_DEBUG_OUTPUT_LEVEL 0x000FFFFF

#else

#define BUS_DEFAULT_DEBUG_OUTPUT_LEVEL 0x0

#endif

#define BUSRESOURCENAME L"MofResourceName"

#ifndef min
#define min(_a, _b)     (((_a) < (_b)) ? (_a) : (_b))
#endif

#ifndef max
#define max(_a, _b)     (((_a) > (_b)) ? (_a) : (_b))
#endif


#define MLX4_MAX_INTERRUPTS		33

typedef struct {
	WDFINTERRUPT				WdfInterrupt;
	BOOLEAN						valid;
} res_interrupt_t;

//
// The device extension of the bus itself.  From whence the PDO's are born.
//

typedef struct _FDO_DEVICE_DATA
{
	BUS_WMI_STD_DATA			WmiData;
	WDFWAITLOCK					ChildLock;
	WDFDEVICE					FdoDevice;
	struct pci_dev 				pci_dev;
	int							card_started;
	int							pci_bus_ifc_taken;
	WDFDMAENABLER				dma_enabler;
	int							dma_adapter_taken;
	MLX4_BUS_IB_INTERFACE		bus_ib_ifc;
	MLX4_FC_BUS_IF		        bus_fc_if;
	int							children_created;
	// Data for the Ethernet device
	struct VipBusIfc			mtnic_Ifc;

#ifndef USE_WDM_INTERRUPTS
	res_interrupt_t 			interrupt[MLX4_MAX_INTERRUPTS];
#endif

} FDO_DEVICE_DATA, *PFDO_DEVICE_DATA;

WDF_DECLARE_CONTEXT_TYPE_WITH_NAME(FDO_DEVICE_DATA, FdoGetData)

//
// The device extension for the PDOs.
// That's of the mxe device which this bus driver enumerates.
//

typedef struct _PDO_DEVICE_DATA
{
	// Unique serial number of the device on the bus
	ULONG 							SerialNo;
	// WDF PDO object
	WDFDEVICE						PdoDevice;
	// FDO context
	PFDO_DEVICE_DATA 				p_fdo;
	// MLX4 BUS IB interface
	WDF_QUERY_INTERFACE_CONFIG		qiMlx4Bus;
	WDF_QUERY_INTERFACE_CONFIG		qiPciBus;
	WDF_QUERY_INTERFACE_CONFIG		qiMlx4FcoxIf;
 
} PDO_DEVICE_DATA, *PPDO_DEVICE_DATA;

WDF_DECLARE_CONTEXT_TYPE_WITH_NAME(PDO_DEVICE_DATA, PdoGetData)


typedef struct _QUEUE_DATA
{
	PFDO_DEVICE_DATA FdoData;

} QUEUE_DATA, *PQUEUE_DATA;

WDF_DECLARE_CONTEXT_TYPE_WITH_NAME(QUEUE_DATA, QueueGetData)

 //
// wmi.c
//

NTSTATUS
WmiRegistration(
	WDFDEVICE      Device
);

NTSTATUS
EvtStdDataSetItem(
	IN  WDFWMIINSTANCE WmiInstance,
	IN  ULONG DataItemId,
	IN  ULONG InBufferSize,
	IN  PVOID InBuffer
	);

NTSTATUS
EvtStdDataSetInstance(
	IN  WDFWMIINSTANCE WmiInstance,
	IN  ULONG InBufferSize,
	IN  PVOID InBuffer
	);

NTSTATUS
EvtStdDataQueryInstance(
	IN  WDFWMIINSTANCE WmiInstance,
	IN  ULONG OutBufferSize,
	IN  PVOID OutBuffer,
	OUT PULONG BufferUsed
	);


//
// drv.c
//

NTSTATUS
DriverEntry(
	IN PDRIVER_OBJECT DriverObject,
	IN PUNICODE_STRING RegistryPath
	);

void
EvtDriverUnload(
	IN		WDFDRIVER  Driver
	);
	
NTSTATUS
EvtDriverDeviceAdd(
	IN WDFDRIVER        Driver,
	IN PWDFDEVICE_INIT  DeviceInit
	);

NTSTATUS
EvtPrepareHardware(
	IN WDFDEVICE  Device,
	IN WDFCMRESLIST  ResourcesRaw,
	IN WDFCMRESLIST  ResourcesTranslated
	);

NTSTATUS
EvtReleaseHardware(
	IN WDFDEVICE  Device,
	IN WDFCMRESLIST  ResourcesTranslated
	);

NTSTATUS
EvtDeviceD0Exit(
	IN WDFDEVICE  Device,
	IN WDF_POWER_DEVICE_STATE  TargetState
	);

NTSTATUS
EvtDeviceD0Entry(
	IN WDFDEVICE  Device,
	IN WDF_POWER_DEVICE_STATE  PreviousState
	);


//
// pci.c
//

ULONG
__find_capability(
    PCI_COMMON_CONFIG* const    pConfig,  
    char cap_id
    );

void
pci_free_msix_info_resources(
	IN struct msix_saved_info * 	pMsixInfo
	);

NTSTATUS
pci_save_config(
	IN				BUS_INTERFACE_STANDARD		*pBusIfc,
		OUT			PCI_COMMON_CONFIG* const	pConfig
	);

NTSTATUS
pci_hca_reset( 
	IN		struct pci_dev *pdev
	);

void
pci_get_uplink_info(
	IN				struct pci_dev *		pdev,
	IN				PCI_COMMON_CONFIG * 	p_cfg,
	OUT 			uplink_info_t * 		p_uplink_info
	);

NTSTATUS
pci_hca_enable(
	IN		PBUS_INTERFACE_STANDARD 	p_ifc,
	IN		PCI_COMMON_CONFIG*			p_cfg
	);

//
// pdo.c
//

NTSTATUS
create_pdo(
	__in WDFDEVICE  Device,
	__in PWCHAR     HardwareIds,
	__in PWCHAR 	CompatibleIds,
	__in PWCHAR     DeviceDescription,
	__in ULONG      SerialNo,
	__in PWCHAR     Location
);

//
// vpd.c
//
NTSTATUS pci_get_vpd(
	BUS_INTERFACE_STANDARD		*pBusIfc,
	PCI_COMMON_CONFIG* const	pConfig,
	UCHAR*						*p_vpd,
	int*						p_vpd_size
	);



