/*++

Copyright (c) 2005-2009 Mellanox Technologies. All rights reserved.

Module Name:
	stat.h

Abstract:
	Statistics Collector header file

Revision History:

Notes:

--*/

#include "precomp.h"

#if defined(EVENT_TRACING)
#ifdef offsetof
#undef offsetof
#endif
#include "stat.tmh"
#endif


MLX4_ST_STAT g_stat;

static void __print_grh( struct mlx4_dev *mdev, struct ib_unpacked_grh *p)
{
    UNUSED_PARAM(mdev);
	MLX4_PRINT(TRACE_LEVEL_VERBOSE, MLX4_DBG_DRV,( "\n\t ========== GRH ==========\n"));
	MLX4_PRINT(TRACE_LEVEL_VERBOSE, MLX4_DBG_DRV,( "\t ip_version        %02x", p->ip_version));
	MLX4_PRINT(TRACE_LEVEL_VERBOSE, MLX4_DBG_DRV,( "\t traffic_class     %02x", p->traffic_class));
	MLX4_PRINT(TRACE_LEVEL_VERBOSE, MLX4_DBG_DRV,( "\t flow_label        %08x", 
		be32_to_cpu(p->flow_label)));
	MLX4_PRINT(TRACE_LEVEL_VERBOSE, MLX4_DBG_DRV,( "\t payload_length    %04x", 
		be16_to_cpu(p->payload_length)));
	MLX4_PRINT(TRACE_LEVEL_VERBOSE, MLX4_DBG_DRV,( "\t next_header       %02x", p->next_header));
	MLX4_PRINT(TRACE_LEVEL_VERBOSE, MLX4_DBG_DRV,( "\t hop_limit         %02x", p->hop_limit));
	MLX4_PRINT(TRACE_LEVEL_VERBOSE, MLX4_DBG_DRV,( "\t source_gid        %08I64x:%08I64x", 
		be64_to_cpu(p->source_gid.global.subnet_prefix),
		be64_to_cpu(p->source_gid.global.interface_id)));
	MLX4_PRINT(TRACE_LEVEL_VERBOSE, MLX4_DBG_DRV,( "\t source_gid        %08I64x:%08I64x", 
		be64_to_cpu(p->destination_gid.global.subnet_prefix),
		be64_to_cpu(p->destination_gid.global.interface_id)));
}

static void __print_deth( struct mlx4_dev *mdev, struct ib_unpacked_deth *p)
{
    UNUSED_PARAM(mdev);
	MLX4_PRINT(TRACE_LEVEL_VERBOSE, MLX4_DBG_DRV,( "\n\t ========== DETH ==========\n"));
	MLX4_PRINT(TRACE_LEVEL_VERBOSE, MLX4_DBG_DRV,( "\t qkey              %08x", 
		be32_to_cpu(p->qkey)));
	MLX4_PRINT(TRACE_LEVEL_VERBOSE, MLX4_DBG_DRV,( "\t source_qpn        %08x", 
		be32_to_cpu(p->source_qpn)));
}

static void __print_bth( struct mlx4_dev *mdev, struct ib_unpacked_bth *p)
{
    UNUSED_PARAM(mdev);
	MLX4_PRINT(TRACE_LEVEL_VERBOSE, MLX4_DBG_DRV,( "\n\t ========== BTH ==========\n"));
	MLX4_PRINT(TRACE_LEVEL_VERBOSE, MLX4_DBG_DRV,( "\t opcode            %02x", p->opcode));
	MLX4_PRINT(TRACE_LEVEL_VERBOSE, MLX4_DBG_DRV,( "\t solicited_event   %02x", p->solicited_event));
	MLX4_PRINT(TRACE_LEVEL_VERBOSE, MLX4_DBG_DRV,( "\t mig_req           %02x", p->mig_req));
	MLX4_PRINT(TRACE_LEVEL_VERBOSE, MLX4_DBG_DRV,( "\t header_version    %02x", p->transport_header_version));
	MLX4_PRINT(TRACE_LEVEL_VERBOSE, MLX4_DBG_DRV,( "\t pkey              %04x", 
		be16_to_cpu(p->pkey)));
	MLX4_PRINT(TRACE_LEVEL_VERBOSE, MLX4_DBG_DRV,( "\t destination_qpn   %08x", 
		be32_to_cpu(p->destination_qpn)));
	MLX4_PRINT(TRACE_LEVEL_VERBOSE, MLX4_DBG_DRV,( "\t ack_req           %02x", p->ack_req));
	MLX4_PRINT(TRACE_LEVEL_VERBOSE, MLX4_DBG_DRV,( "\t psn               %08x", 
		be32_to_cpu(p->psn)));
}

static void __print_lrh( struct mlx4_dev *mdev, struct ib_unpacked_lrh *p)
{
    UNUSED_PARAM(mdev);
	MLX4_PRINT(TRACE_LEVEL_VERBOSE, MLX4_DBG_DRV,( "\n\t ========== LRH ==========\n"));
	MLX4_PRINT(TRACE_LEVEL_VERBOSE, MLX4_DBG_DRV,( "\t virtual_lane      %02x", p->virtual_lane));
	MLX4_PRINT(TRACE_LEVEL_VERBOSE, MLX4_DBG_DRV,( "\t link_version      %02x", p->link_version));
	MLX4_PRINT(TRACE_LEVEL_VERBOSE, MLX4_DBG_DRV,( "\t service_level     %02x", p->service_level));
	MLX4_PRINT(TRACE_LEVEL_VERBOSE, MLX4_DBG_DRV,( "\t link_next_header  %02x", p->link_next_header));
	MLX4_PRINT(TRACE_LEVEL_VERBOSE, MLX4_DBG_DRV,( "\t destination_lid   %04x", 
		be16_to_cpu(p->destination_lid)));
	MLX4_PRINT(TRACE_LEVEL_VERBOSE, MLX4_DBG_DRV,( "\t packet_length     %04x", 
		be16_to_cpu(p->packet_length)));
	MLX4_PRINT(TRACE_LEVEL_VERBOSE, MLX4_DBG_DRV,( "\t source_lid        %04x", 
		be16_to_cpu(p->source_lid)));
}

static void __print_ud_header( struct mlx4_dev *mdev, struct ib_ud_header *p)
{
	MLX4_PRINT(TRACE_LEVEL_VERBOSE, MLX4_DBG_DRV,( "\n\t ========== UD HEADER ==========\n"));

	MLX4_PRINT(TRACE_LEVEL_VERBOSE, MLX4_DBG_DRV,( "\t grh_present %d, imm_present %d, imm %08x",
		p->grh_present, p->immediate_present, be32_to_cpu(p->immediate_data) ));

	if ( mdev->pdev->p_stat->flags & MLX4_MAD_TRACE_LRH )
		__print_lrh( mdev, &p->lrh );

	if ( mdev->pdev->p_stat->flags & MLX4_MAD_TRACE_BTH )
		__print_bth( mdev, &p->bth );
	
	if ( mdev->pdev->p_stat->flags & MLX4_MAD_TRACE_DETH )
		__print_deth( mdev, &p->deth );
	
	if ( p->grh_present && (mdev->pdev->p_stat->flags & MLX4_MAD_TRACE_GRH) )
		__print_grh( mdev, &p->grh );
	
}

static void __print_mlx( struct mlx4_dev *mdev, struct mlx4_wqe_mlx_seg *p)
{
    UNUSED_PARAM(mdev);
	MLX4_PRINT(TRACE_LEVEL_VERBOSE, MLX4_DBG_DRV,( "\n\t ========== MLX WQE ==========\n"));
	MLX4_PRINT(TRACE_LEVEL_VERBOSE, MLX4_DBG_DRV,( "\t owner             %02x", p->owner));
	MLX4_PRINT(TRACE_LEVEL_VERBOSE, MLX4_DBG_DRV,( "\t opcode            %02x", p->opcode));
	MLX4_PRINT(TRACE_LEVEL_VERBOSE, MLX4_DBG_DRV,( "\t size              %02x", p->size));
	MLX4_PRINT(TRACE_LEVEL_VERBOSE, MLX4_DBG_DRV,( "\t flags             %08x", 
		be32_to_cpu(p->flags)));
	MLX4_PRINT(TRACE_LEVEL_VERBOSE, MLX4_DBG_DRV,( "\t rlid              %04x", 
		be16_to_cpu(p->rlid)));
}

void st_print_mlx_header( struct mlx4_dev *mdev, struct mlx4_ib_sqp *sqp, struct mlx4_wqe_mlx_seg *mlx )
{
	if ( mdev->pdev->p_stat->flags & MLX4_MAD_TRACE_UDH )
		__print_ud_header( mdev, &sqp->hdr.ib );
	if ( mdev->pdev->p_stat->flags & MLX4_MAD_TRACE_WQE )
		__print_mlx( mdev, mlx );
}

void st_print_mlx_send(struct mlx4_dev *mdev, struct ib_qp *ibqp, ib_send_wr_t *wr)
{
	struct mlx4_ib_qp *qp = to_mqp(ibqp);

	if ( mdev->pdev->p_stat->flags & MLX4_MAD_TRACE_WR ) {
		MLX4_PRINT(TRACE_LEVEL_VERBOSE, MLX4_DBG_DRV,( "\n\t ========== SEND WR on QP %#x (%#x) ==========\n",
			ibqp->qp_num, qp->mqp.qpn ));
		MLX4_PRINT(TRACE_LEVEL_VERBOSE, MLX4_DBG_DRV,( "\t wr_type           %d", wr->wr_type));
		MLX4_PRINT(TRACE_LEVEL_VERBOSE, MLX4_DBG_DRV,( "\t wr_id             %08I64x", wr->wr_id));
		MLX4_PRINT(TRACE_LEVEL_VERBOSE, MLX4_DBG_DRV,( "\t send_opt          %x", wr->send_opt));
		MLX4_PRINT(TRACE_LEVEL_VERBOSE, MLX4_DBG_DRV,( "\t immediate_data    %x", be32_to_cpu(wr->immediate_data)));
		MLX4_PRINT(TRACE_LEVEL_VERBOSE, MLX4_DBG_DRV,( "\t num_ds            %d", wr->num_ds));
		MLX4_PRINT(TRACE_LEVEL_VERBOSE, MLX4_DBG_DRV,( "\t ds[0].pa          %I64x", wr->ds_array[0].vaddr));
		MLX4_PRINT(TRACE_LEVEL_VERBOSE, MLX4_DBG_DRV,( "\t ds[0].length      %x", wr->ds_array[0].length));
		MLX4_PRINT(TRACE_LEVEL_VERBOSE, MLX4_DBG_DRV,( "\t ds[0].lkey        %x", wr->ds_array[0].lkey));
	}
}

void st_dump_mlx_wqe(struct mlx4_dev *mdev, void *wqe, int size_in_dwords, ib_send_wr_t *wr)
{
	int j;
	u32 *ptr = wqe;
#if 0	
	int i, size;
#else	
	UNUSED_PARAM(wr);
#endif
	
	if ( mdev->pdev->p_stat->flags & MLX4_MAD_TRACE_MLX_WQE_DUMP ) {

		MLX4_PRINT(TRACE_LEVEL_VERBOSE, MLX4_DBG_DRV,( "\n\t ========== MLX WQE at %p, size %#x ==========\n",
			wqe, size_in_dwords*4 ));
		
		for ( j = 0; j < size_in_dwords; ++j ) {
			MLX4_PRINT(TRACE_LEVEL_WARNING, MLX4_DBG_DRV,( "\t %04x:	%08x %08x %08x %08x \n", 16*j, 
				be32_to_cpu(ptr[4*j]), be32_to_cpu(ptr[4*j + 1]),
				be32_to_cpu(ptr[4*j + 2]),be32_to_cpu(ptr[4*j + 3]) ));
		}
	}

#if 0
	for ( j = 0; j < (int)wr->num_ds; ++j ) {
		mlx4_dbg(mdev, "\n\t ========== SMP %d at pa %I64x, size %#x, lkey %#x ==========\n",
			j, wr->ds_array[0].vaddr, wr->ds_array[0].length, wr->ds_array[0].lkey );

		//TODO: vaddr should be converted to virtual address
		ptr = (PVOID)(ULONG_PTR)wr->ds_array[0].vaddr;
		size = (wr->ds_array[0].length + 3) >> 2;
		for ( i = 0; i < size; ++i ) {
			mlx4_warn( mdev, "%04x:	%08x %08x %08x %08x \n", 16*i, 
				be32_to_cpu(ptr[4*i]), be32_to_cpu(ptr[4*i + 1]),
				be32_to_cpu(ptr[4*i + 2]),be32_to_cpu(ptr[4*i + 3]) );
		}
	}
#endif	

}

void st_dev_add_cont_mem_stat( PMLX4_ST_DEVICE p_stat, ULONG size )
{
	if (p_stat) {
		p_stat->n_cont_allocs++;
		p_stat->total_cont_allocs += size;
	}
}

void st_dev_rmv_cont_mem_stat( PMLX4_ST_DEVICE p_stat, ULONG size )
{
	if (p_stat) {
		p_stat->n_cont_frees++;
		p_stat->total_cont_frees += size;
	}
}

void st_dev_add_thread( PMLX4_ST_DEVICE p_stat, char *name )
{
	if ( p_stat ) {
		if ( name ) {
			RtlStringCbCopyA( p_stat->thread_comm.thread_name, 
				sizeof(p_stat->thread_comm.thread_name), name );
			p_stat->thread_comm.p_thread = KeGetCurrentThread();
		}
		else {
			p_stat->thread_comm.p_thread = NULL;
			p_stat->thread_comm.thread_name[0] = '\0';
		}
	}
}

void st_dev_rmv( PMLX4_ST_DEVICE p_stat )
{
	if ( p_stat && p_stat->valid ) {
		p_stat->valid = FALSE;
		if ( p_stat->p_card->n_slaves-- == 1 ) {
			p_stat->p_card->valid = FALSE;
			g_stat.drv.n_cards--;
		}
	}
}

PMLX4_ST_DEVICE st_dev_get(int bus, int func)
{
	PMLX4_ST_CARD p_card = NULL;
	int i, found = FALSE;

	// sanity check
	if (func >= MLX4_ST_MAX_DEVICES) {
		cl_msg_out( "Incorrect function number %d (max is %d) \n", func, MLX4_ST_MAX_DEVICES );
		return NULL;
	}

	// look for a card slot in existing ones
	for ( i = 0 ; i < MLX4_ST_MAX_CARDS; ++i ) {
		p_card = &g_stat.card[i];
		if ( p_card->valid ) {
			if ( p_card->bus_num != bus )
				continue;
			found = TRUE;
			break;
		}
	}

	if ( found ) 
		return &p_card->dev[func];
	else
		return NULL;
}

PMLX4_ST_DEVICE st_dev_add(int bus, int func)
{
	PMLX4_ST_DEVICE p_stat;
	PMLX4_ST_CARD p_card = NULL;
	int i, found = FALSE;

	// sanity check
	if (func >= MLX4_ST_MAX_DEVICES) {
		cl_msg_out( "Incorrect function number %d (max is %d) \n", func, MLX4_ST_MAX_DEVICES );
		return NULL;
	}

	// look for a card slot in existing
	for ( i = 0 ; i < MLX4_ST_MAX_CARDS; ++i ) {
		p_card = &g_stat.card[i];
		if ( p_card->valid ) {
			if ( p_card->bus_num != bus )
				continue;
			found = TRUE;
			break;
		}
	}

	// allocate a new card slot
	if ( !found ) {
		for ( i = 0 ; i < MLX4_ST_MAX_CARDS; ++i ) {
			p_card = &g_stat.card[i];
			if ( !p_card->valid ) {
				p_card->valid = TRUE;
				p_card->bus_num = bus;
				g_stat.drv.n_cards++;
				found = TRUE;
				break;
			}
		}
	}		

	// failed card slot search
	if ( !found ) {
		cl_msg_out( "Not found place for globals statistics: n_cards %d (max is %d, bus %d, func %d) \n", 
			g_stat.drv.n_cards, MLX4_ST_MAX_CARDS, bus, func );
		return NULL;
	}	

	// find device slot
	p_stat = &p_card->dev[func];
	ASSERT( p_stat->valid == FALSE );
	memset( p_stat, 0, sizeof(p_card->dev[func]));
	p_stat->valid = TRUE;
	p_stat->func_num = (USHORT)func;
	p_stat->p_card = p_card;
	p_card->n_slaves++;

	return p_stat;
}


