/*
 * Copyright (c) 2005 SilverStorm Technologies.  All rights reserved.
 * Copyright (c) 1996-2003 Intel Corporation. All rights reserved. 
 *
 * This software is available to you under the OpenIB.org BSD license
 * below:
 *
 *     Redistribution and use in source and binary forms, with or
 *     without modification, are permitted provided that the following
 *     conditions are met:
 *
 *      - Redistributions of source code must retain the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer.
 *
 *      - Redistributions in binary form must reproduce the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer in the documentation and/or other materials
 *        provided with the distribution.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 * $Id: al.c 1611 2006-08-20 14:48:55Z sleybo $
 */


#pragma once

////////////////////////////////////////////////////////
//
// INCLUDES
//
////////////////////////////////////////////////////////

#ifdef USE_WDM_FRAMEWORK
#include <complib/cl_pnp_po.h>
#endif
#include <iba/ib_ci_ifc.h>
#include "data.h"
#include "debug.h"
#include "bus_intf.h"


////////////////////////////////////////////////////////
//
// MACROS
//
////////////////////////////////////////////////////////

#if !defined(FILE_DEVICE_INFINIBAND) // Not defined in WXP DDK
#define FILE_DEVICE_INFINIBAND          0x0000003B
#endif

#define BUSENUM_POOL_TAG (ULONG) 'suBT'

#define HCARESOURCENAME L"MofResourceName"

#ifndef min
#define min(_a, _b)     (((_a) < (_b)) ? (_a) : (_b))
#endif

#ifndef max
#define max(_a, _b)     (((_a) > (_b)) ? (_a) : (_b))
#endif


////////////////////////////////////////////////////////
//
// TYPES
//
////////////////////////////////////////////////////////

/****s* HCA/hca_reg_state_t
* NAME
*	hca_reg_state_t
*
* DESCRIPTION
*	State for tracking registration with AL.  This state is independent of the
*	device PnP state, and both are used to properly register with AL.
*
* SYNOPSIS
*/
typedef enum _hca_reg_state
{
	HCA_SHUTDOWN,
	HCA_ADDED,
	HCA_STARTED

}	hca_reg_state_t;
/*
* VALUES
*	HCA_SHUTDOWN
*		Cleaning up.
*
*	HCA_ADDED
*		AddDevice was called and successfully registered for interface
*		notifications.
*
*	HCA_STARTED
*		IRP_MN_START_DEVICE was called.  The HCA is fully functional.
*
*********/

//
// Structure for reporting data to WMI
//

typedef struct _HCA_WMI_STD_DATA {
	UINT32 DebugPrintLevel;
	UINT32 DebugPrintFlags;

} HCA_WMI_STD_DATA, * PHCA_WMI_STD_DATA;


#pragma warning(disable:4201) // nameless struct/union
typedef struct _GLOBALS {
	HCA_WMI_STD_DATA;
} GLOBALS;
#pragma warning(default:4201) // nameless struct/union

extern GLOBALS g;

//
// The device extension of the bus itself.  From whence the PDO's are born.
//

typedef struct _FDO_DEVICE_DATA
{
#ifdef USE_WDM_FRAMEWORK
	/* -------------------------------------------------
	*		PNP and POWER MANAGER DATA	 
	* ------------------------------------------------ */
	cl_pnp_po_ext_t	cl_ext;						/* COMPLIB PnP object */
	/* Cache of the system to device power states. */
	DEVICE_POWER_STATE		DevicePower[PowerSystemMaximum];
	DEVICE_POWER_STATE		DevicePowerState;
	SYSTEM_POWER_STATE		SystemPowerState;
	PIO_WORKITEM			pPoWorkItem;

#else

	/* -------------------------------------------------
	*		WDF DATA	 
	* ------------------------------------------------ */
	WDFDEVICE					FdoDevice;
	DEVICE_OBJECT			*	p_dev_obj;		/* WDM dev object */
#endif

	HCA_WMI_STD_DATA			WmiData;

#ifdef USE_WDM_FRAMEWORK
	/* -------------------------------------------------
	*		PNP DATA 	 
	* ------------------------------------------------ */
	void					*	pnp_ifc_entry;			/* Notification entry for PnP interface events. */
	void					*	pnp_target_entry;	/* Notification entry for PnP target events. */
	PNP_DEVICE_STATE			pnpState; /* state for PnP Manager */
#endif	

	/* -------------------------------------------------
	*		IBAL DATA 	 
	* ------------------------------------------------ */
	/* Number of references on the upper interface. */
	atomic32_t					n_hca_ifc_ref;
	hca_reg_state_t				state;				/* State for tracking registration with AL */
#ifdef USE_WDM_FRAMEWORK
	DEVICE_OBJECT			*	p_al_dev;		/* IB_AL FDO */
	FILE_OBJECT				*	p_al_file_obj;	/* IB_AL file object */
	UNICODE_STRING				al_sym_name;	/* IB_AL symbolic name */
#endif	

	/* -------------------------------------------------
	*		SHIM DATA	 
	* ------------------------------------------------ */
	mlnx_hca_t					hca;
	atomic32_t					usecnt; /* the number of working applications*/
	spinlock_t					uctx_lock;			// spinlock for the below chain
	cl_qlist_t					uctx_list;			// chain of user contexts
	int							bus_pci_ifc_taken;
	BUS_INTERFACE_STANDARD		bus_pci_ifc;	/* PCI bus interface */

	/* -------------------------------------------------
	*		LOW LEVEL DRIVER' DATA	 
	* ------------------------------------------------ */
	int							bus_ib_ifc_taken;
	MLX4_BUS_IB_INTERFACE		bus_ib_ifc;


} FDO_DEVICE_DATA, *PFDO_DEVICE_DATA;

#ifndef USE_WDM_FRAMEWORK

WDF_DECLARE_CONTEXT_TYPE_WITH_NAME(FDO_DEVICE_DATA, FdoGetData)

typedef struct _QUEUE_DATA
{
	PFDO_DEVICE_DATA FdoData;

} QUEUE_DATA, *PQUEUE_DATA;

WDF_DECLARE_CONTEXT_TYPE_WITH_NAME(QUEUE_DATA, QueueGetData)

#endif

////////////////////////////////////////////////////////
//
// FUNCTIONS
//
////////////////////////////////////////////////////////

static inline PFDO_DEVICE_DATA hca2fdo(mlnx_hca_t *p_hca)
{
	return CONTAINING_RECORD(p_hca, FDO_DEVICE_DATA, hca);
}

static inline struct ib_device *hca2ibdev(mlnx_hca_t *p_hca)
{
	return (hca2fdo(p_hca))->bus_ib_ifc.p_ibdev;
}

static inline mlnx_hca_t *ibdev2hca(struct ib_device *p_ibdev)
{
	return &p_ibdev->x.p_fdo->hca;
}

static inline struct pci_dev *hca2pdev(mlnx_hca_t *p_hca)
{
	return (hca2fdo(p_hca))->bus_ib_ifc.pdev;
}

static inline struct mlx4_dev *hca2mdev(mlnx_hca_t *p_hca)
{
	return (hca2fdo(p_hca))->bus_ib_ifc.pdev->dev;
}

static inline ib_api_status_t errno_to_iberr(int err)
{
#define MAP_NT_ERR(err,ibstatus)	case err: ib_status = ibstatus; break
	ib_api_status_t ib_status;

	if (!err) 
		return IB_SUCCESS;

	if (err < 0)
		err = -err;
	switch (err) {
		MAP_NT_ERR( ENOENT, IB_NOT_FOUND );
		MAP_NT_ERR( EINTR, IB_INTERRUPTED );
		MAP_NT_ERR( EAGAIN, IB_RESOURCE_BUSY );
		MAP_NT_ERR( ENOMEM, IB_INSUFFICIENT_MEMORY );
		MAP_NT_ERR( EACCES, IB_INVALID_PERMISSION );
		MAP_NT_ERR( EFAULT, IB_ERROR );
		MAP_NT_ERR( EBUSY, IB_RESOURCE_BUSY );
		MAP_NT_ERR( ENODEV, IB_UNSUPPORTED );
		MAP_NT_ERR( EINVAL, IB_INVALID_PARAMETER );
		MAP_NT_ERR( ENOSYS, IB_UNSUPPORTED );
		MAP_NT_ERR( ERANGE, IB_INVALID_SETTING );
		default:
			//HCA_PRINT(TRACE_LEVEL_ERROR, HCA_DBG_SHIM,
			//	"Unmapped errno (%d)\n", err);
			ib_status = IB_UNKNOWN_ERROR;
			break;
	}
	return ib_status;
}

static inline int start_port(struct ib_device *device)
{
	return device->node_type == RDMA_NODE_IB_SWITCH ? 0 : 1;
}

static inline int end_port(struct ib_device *device)
{
	return device->node_type == RDMA_NODE_IB_SWITCH ? 0 : device->phys_port_cnt;
}

#ifndef USE_WDM_FRAMEWORK

////////////////////////////////////////////////////////
//
// PROTOTYPES
//
////////////////////////////////////////////////////////

DRIVER_INITIALIZE DriverEntry;

void
EvtDriverUnload(
	IN		WDFDRIVER  Driver
	);

NTSTATUS
EvtDriverDeviceAdd(
	IN WDFDRIVER Driver,
	IN PWDFDEVICE_INIT Device
	);

NTSTATUS
EvtDeviceD0Entry(
	IN WDFDEVICE  Device,
	IN WDF_POWER_DEVICE_STATE  PreviousState
	);

NTSTATUS
EvtDeviceD0Exit(
	IN WDFDEVICE  Device,
	IN WDF_POWER_DEVICE_STATE  TargetState
	);

NTSTATUS
EvtPrepareHardware(
	IN WDFDEVICE  Device,
	IN WDFCMRESLIST  ResourcesRaw,
	IN WDFCMRESLIST  ResourcesTranslated
	);

NTSTATUS
EvtReleaseHardware(
	IN WDFDEVICE  Device,
	IN WDFCMRESLIST  ResourcesTranslated
	);


//
// Implemented in wmi.c
//

NTSTATUS
WmiRegistration(
	WDFDEVICE      Device
);

NTSTATUS
EvtStdDataSetItem(
	IN  WDFWMIINSTANCE WmiInstance,
	IN  ULONG DataItemId,
	IN  ULONG InBufferSize,
	IN  PVOID InBuffer
	);

NTSTATUS
EvtStdDataSetInstance(
	IN  WDFWMIINSTANCE WmiInstance,
	IN  ULONG InBufferSize,
	IN  PVOID InBuffer
	);

NTSTATUS
EvtStdDataQueryInstance(
	IN  WDFWMIINSTANCE WmiInstance,
	IN  ULONG OutBufferSize,
	IN  PVOID OutBuffer,
	OUT PULONG BufferUsed
	);

#endif

